﻿using System;
using System.Collections.Generic;

using Microsoft.Office.Server.Search.Administration;

using Common;

namespace Layers
{
    public class O12Conversions
    {
        public static TKeyword ToKeyword(Keyword key)
        {
            TKeyword newKey = new TKeyword();
            newKey.Contact = key.Contact;
            newKey.Definition = key.Definition;
            newKey.EndDate = key.EndDate;
            newKey.ReviewDate = key.ReviewDate;
            newKey.StartDate = key.StartDate;
            newKey.Term = key.Term;

            newKey.Bestbet = new TBestbet[key.BestBets.Count];
            int i = 0;
            foreach(BestBet bb in key.BestBets)
            {
                newKey.Bestbet[i] = ToBestbet(bb);
                i++;
            }

            newKey.Synonym = new String[key.Synonyms.Count];
            i = 0;
            foreach (Synonym syn in key.Synonyms)
            {
                newKey.Synonym[i] = syn.Term;
                i++;
            }

            return newKey;
        }

        public static TBestbet ToBestbet(BestBet bb)
        {
            TBestbet newBB = new TBestbet();
            newBB.Description = bb.Description;
            newBB.Title = bb.Title;
            newBB.Url = bb.Url.ToString();
            return newBB;
        }

        public static TScope ToScope(Scope scope)
        {
            TScope newScope = new TScope();
            newScope.Name = scope.Name;
            newScope.AlternateResultsPage = scope.AlternateResultsPage;
            newScope.CompilationType = Util.MarshalEnum<ScopeCompilationType, TScopeCompilation>(scope.CompilationType);
            newScope.Description = scope.Description;
            newScope.ScopeKey = scope.ID.ToString();

            List<TAllContentScopeRule> allContentScopeRuleList;
            List<TUrlScopeRule> urlScopeRuleList;
            List<TPropertyQueryScopeRule> propertyQueryScopeRuleList;

            BucketRules(scope.Rules, out allContentScopeRuleList, out urlScopeRuleList, out propertyQueryScopeRuleList);
            newScope.AllContentScopeRule = allContentScopeRuleList.ToArray();
            newScope.UrlScopeRule = urlScopeRuleList.ToArray();
            newScope.PropertyQueryScopeRule = propertyQueryScopeRuleList.ToArray();
            return newScope;
        }

        public static TScopeDisplayGroup ToScopeDisplayGroup(ScopeDisplayGroup group)
        {
            TScopeDisplayGroup newGroup = new TScopeDisplayGroup();
            newGroup.Name = group.Name;
            newGroup.Description = group.Description;
            newGroup.DefaultScopeKey = (null != group.Default ? group.Default.ID.ToString() : null);

            newGroup.ScopeKey = new String[group.Count];
            int i = 0;
            foreach (Scope scope in group)
            {
                newGroup.ScopeKey[i] = scope.ID.ToString();
                i++;
            }
            return newGroup;
        }

        public static TUrlScopeRule ToUrlScopeRule(UrlScopeRule rule)
        {
            TUrlScopeRule newRule = new TUrlScopeRule();
            newRule.FilterBehavior = Util.MarshalEnum<ScopeRuleFilterBehavior, TFilterBehavior>(rule.FilterBehavior);
            newRule.UrlScopeRuleType = Util.MarshalEnum<UrlScopeRuleType, TUrlScopeRuleType>(rule.UrlRuleType);
            newRule.MatchingString = rule.MatchingString;
            return newRule;
        }

        public static TPropertyQueryScopeRule ToPropertyQueryScopeRule(PropertyQueryScopeRule rule)
        {
            TPropertyQueryScopeRule newRule = new TPropertyQueryScopeRule();
            newRule.FilterBehavior = Util.MarshalEnum<ScopeRuleFilterBehavior, TFilterBehavior>(rule.FilterBehavior);
            newRule.Property = rule.Property.Name;
            newRule.Value = rule.Value;
            return newRule;
        }

        private static void BucketRules(ScopeRuleCollection rules,
           out List<TAllContentScopeRule> allContentScopeRuleList,
           out List<TUrlScopeRule> urlScopeRuleList,
           out List<TPropertyQueryScopeRule> propertyQueryScopeRuleList)
        {
            allContentScopeRuleList = new List<TAllContentScopeRule>();
            urlScopeRuleList = new List<TUrlScopeRule>();
            propertyQueryScopeRuleList = new List<TPropertyQueryScopeRule>();

            foreach (ScopeRule rule in rules)
            {
                if (rule is AllContentScopeRule)
                {
                    allContentScopeRuleList.Add(new TAllContentScopeRule());
                }
                else if (rule is UrlScopeRule)
                {
                    urlScopeRuleList.Add(ToUrlScopeRule(rule as UrlScopeRule));
                }
                else if (rule is PropertyQueryScopeRule)
                {
                    propertyQueryScopeRuleList.Add(ToPropertyQueryScopeRule(rule as PropertyQueryScopeRule));
                }
            }
        }
    }
}